#pragma once

#include <lua.hpp>

namespace Bix
{


	class State {
	public:
		lua_State *_l;
	public:
		State() : _l(nullptr) { _l = luaL_newstate(); }
		State(const State &other) = delete;
		State &operator=(const State &other) = delete;
		State(State &&other) : _l(other._l) { other._l = nullptr; }
		~State() { if (_l == nullptr) return; lua_close(_l); }

		template<typename T> void Push(const T value);
		template<typename T> T Read(int index);

		template <typename T, typename... Ts>
		int Push(const T value, const Ts... values) {
			Push<T>(value);
			Push(values...);
			return sizeof...(values) + 1;
		}

		void Push(){}




	};

	template<> void State::Push(const int value)			{ lua_pushinteger(_l, value); }
	template<> void State::Push(const float value)			{ lua_pushnumber(_l, value); }
	template<> void State::Push(const double value)			{ lua_pushnumber(_l, value); }
	template<> void State::Push(const std::string value)	{ lua_pushstring(_l, value.c_str()); }
	template<> void State::Push(const char *value)			{ lua_pushstring(_l, value); }

	template<> float		State::Read(int index)				{ return (float)lua_tonumber(_l, index); }
	template<> bool			State::Read(int index)				{ return (bool)lua_toboolean(_l, index); }
	template<> std::string	State::Read(int index)				{ return std::string(lua_tostring(_l, index)); }



}
